@@ -45,7 +45,7 @@ typedef struct
     gint dpi;
     ScanMode scan_mode;
     gint depth;
-    gboolean multi_page;
+    gboolean type;
 } ScanJob;
 
 typedef struct
@@ -423,16 +423,18 @@ control_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_I
 }
 
 
-static void
+static gboolean
 set_default_option (SANE_Handle handle, const SANE_Option_Descriptor *option, SANE_Int option_index)
 {
     SANE_Status status;
-    
+
     status = sane_control_option (handle, option_index, SANE_ACTION_SET_AUTO, NULL, NULL);
     g_debug ("sane_control_option (%d, SANE_ACTION_SET_AUTO) -> %s",
              option_index, get_status_string (status));
     if (status != SANE_STATUS_GOOD)
         g_warning ("Error setting default option %s: %s", option->name, sane_strstatus(status));
+
+    return status == SANE_STATUS_GOOD;
 }
 
 
@@ -929,22 +931,68 @@ do_get_option (Scanner *scanner)
         }
     }
     else if (strcmp (option->name, SANE_NAME_SCAN_SOURCE) == 0) {
+        const char *flatbed_sources[] =
+        {
+            "Flatbed",
+            SANE_I18N ("Flatbed"),
+            "FlatBed",
+            "Normal",
+            SANE_I18N ("Normal"),
+            NULL
+        };
+
         const char *adf_sources[] =
         {
             "Automatic Document Feeder",
             SANE_I18N ("Automatic Document Feeder"),
             "ADF",
-            SANE_I18N ("ADF Front"),
-            "ADF Front",
+            "Automatic Document Feeder(left aligned)", /* Seen in the proprietary brother3 driver */
+            "Automatic Document Feeder(centrally aligned)", /* Seen in the proprietary brother3 driver */
             NULL
         };
 
-        if (job->multi_page) {
-            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))
-                 g_warning ("Unable to set ADF source, please file a bug");
-        }
-        else {
-            set_default_option (scanner->priv->handle, option, option_index);
+        const char *adf_front_sources[] =
+        {
+            "ADF Front",
+            SANE_I18N ("ADF Front"),
+            NULL
+        };
+
+        const char *adf_back_sources[] =
+        {
+            "ADF Back",
+            SANE_I18N ("ADF Back"),
+            NULL
+        };
+
+        const char *adf_duplex_sources[] =
+        {
+            "ADF Duplex",
+            SANE_I18N ("ADF Duplex"),
+            NULL
+        };
+
+        switch (job->type) {
+        case SCAN_SINGLE:
+            if (!set_default_option (scanner->priv->handle, option, option_index))
+                if (!set_constrained_string_option (scanner->priv->handle, option, option_index, flatbed_sources))
+                    g_warning ("Unable to set single page source, please file a bug");
+            break;
+        case SCAN_ADF_FRONT:
+            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_front_sources))
+                if (!!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))                
+                    g_warning ("Unable to set front ADF source, please file a bug");
+            break;
+        case SCAN_ADF_BACK:
+            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_back_sources))
+                if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))
+                    g_warning ("Unable to set back ADF source, please file a bug");
+            break;
+        case SCAN_ADF_BOTH:
+            if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_duplex_sources))
+                if (!set_constrained_string_option (scanner->priv->handle, option, option_index, adf_sources))
+                    g_warning ("Unable to set duplex ADF source, please file a bug");
+            break;
         }
     }
     else if (strcmp (option->name, SANE_NAME_BIT_DEPTH) == 0) {
@@ -957,6 +1005,7 @@ do_get_option (Scanner *scanner)
         {
             SANE_VALUE_SCAN_MODE_COLOR,
             "Color",
+            "24bit Color", /* Seen in the proprietary brother3 driver */
             NULL
         };
         const char *gray_scan_modes[] =
@@ -1170,7 +1219,7 @@ do_complete_page (Scanner *scanner)
     }
 
     /* Go back for another page */
-    if (job->multi_page) {
+    if (job->type != SCAN_SINGLE) {
         scanner->priv->page_number++;
         scanner->priv->pass_number = 0;
         emit_signal (scanner, PAGE_DONE, NULL);
@@ -1370,11 +1419,30 @@ scanner_is_scanning (Scanner *scanner)
 
 void
 scanner_scan (Scanner *scanner, const char *device,
-              gint dpi, ScanMode scan_mode, gint depth, gboolean multi_page)
+              gint dpi, ScanMode scan_mode, gint depth, ScanType type)
 {
     ScanRequest *request;
+    const gchar *type_string;
+  
+    switch (type) {
+    case SCAN_SINGLE:
+        type_string = "SCAN_SINGLE";
+        break;
+    case SCAN_ADF_FRONT:
+        type_string = "SCAN_ADF_FRONT";
+        break;
+    case SCAN_ADF_BACK:
+        type_string = "SCAN_ADF_BACK";
+        break;
+    case SCAN_ADF_BOTH:
+        type_string = "SCAN_ADF_BOTH";
+        break;
+    default:
+        g_assert (FALSE);
+        return;
+    }
 
-    g_debug ("scanner_scan (\"%s\", %d, %s)", device ? device : "(null)", dpi, multi_page ? "TRUE" : "FALSE");
+    g_debug ("scanner_scan (\"%s\", %d, %s)", device ? device : "(null)", dpi, type_string);
     request = g_malloc0 (sizeof (ScanRequest));
     request->type = REQUEST_START_SCAN;
     request->job = g_malloc0 (sizeof (ScanJob));
@@ -1382,7 +1450,7 @@ scanner_scan (Scanner *scanner, const char *device,
     request->job->dpi = dpi;
     request->job->scan_mode = scan_mode;
     request->job->depth = depth;
-    request->job->multi_page = multi_page;
+    request->job->type = type;
     g_async_queue_push (scanner->priv->scan_queue, request);
 }
 
