@@ -15,11 +15,16 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <math.h>
+#include <unistd.h> // TEMP: Needed for close() in get_temporary_filename()
 
 #include "ui.h"
 #include "book-view.h"
 
 
+#define DEFAULT_TEXT_DPI 150
+#define DEFAULT_PHOTO_DPI 300
+
+
 enum {
     START_SCAN,
     STOP_SCAN,
@@ -42,14 +47,16 @@ struct SimpleScanPrivate
     GtkTreeModel *mode_model;
     GtkWidget *preview_box, *preview_area, *preview_scroll;
     GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
+    GtkWidget *stop_menuitem, *stop_toolbutton;
 
     GtkWidget *authorize_dialog;
     GtkWidget *authorize_label;
     GtkWidget *username_entry, *password_entry;
 
     GtkWidget *preferences_dialog;
-    GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo;
-    GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model;
+    GtkWidget *device_combo, *text_dpi_combo, *photo_dpi_combo, *page_side_combo;
+    GtkTreeModel *device_model, *text_dpi_model, *photo_dpi_model, *page_side_model;
+    gboolean setting_devices, user_selected_device;
 
     Book *book;
     BookView *book_view;
@@ -133,13 +140,82 @@ ui_authorize (SimpleScan *ui, const gchar *resource, gchar **username, gchar **p
     *password = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->password_entry)));
 }
 
+
+void device_combo_changed_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+device_combo_changed_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (ui->priv->setting_devices)
+        return;
+    ui->priv->user_selected_device = TRUE;
+}
+
+
 void
 ui_set_scan_devices (SimpleScan *ui, GList *devices)
 {
+    GList *d;
+    gboolean have_selection = FALSE;
+    gint index;
     GtkTreeIter iter;
-    GList *i;
-    gboolean have_iter;
-    gboolean have_selection;
+  
+    ui->priv->setting_devices = TRUE;
+ 
+    /* If the user hasn't chosen a scanner choose the best available one */
+    if (ui->priv->user_selected_device)
+        have_selection = gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) >= 0;
+
+    /* Add new devices */
+    index = 0;
+    for (d = devices; d; d = d->next) {
+        ScanDevice *device = (ScanDevice *) d->data;
+        gint n_delete = -1;
+
+        /* Find if already exists */
+        if (gtk_tree_model_iter_nth_child (ui->priv->device_model, &iter, NULL, index)) {
+            gint i = 0;
+            do {
+                gchar *name;
+                gboolean matched;
+
+                gtk_tree_model_get (ui->priv->device_model, &iter, 0, &name, -1);
+                matched = strcmp (name, device->name) == 0;
+                g_free (name);
+
+                if (matched) {
+                    n_delete = i;
+                    break;
+                }
+                i++;
+            } while (gtk_tree_model_iter_next (ui->priv->device_model, &iter));
+        }
+      
+        /* If exists, remove elements up to this one */
+        if (n_delete >= 0) {
+            gint i;
+
+            for (i = 0; i < n_delete; i++) {
+                gtk_tree_model_iter_nth_child (ui->priv->device_model, &iter, NULL, index);
+                gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
+            }
+        }
+        else {
+            gtk_list_store_insert (GTK_LIST_STORE (ui->priv->device_model), &iter, index);
+            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device->name, 1, device->label, -1);
+        }
+        index++;
+    }
+
+    /* Remove any remaining devices */
+    while (gtk_tree_model_iter_nth_child (ui->priv->device_model, &iter, NULL, index))
+        gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
+
+    /* Select the first available device */
+    if (!have_selection && devices != NULL)
+        gtk_combo_box_set_active (GTK_COMBO_BOX (ui->priv->device_combo), 0);
+
+    ui->priv->setting_devices = FALSE;
 
     if (!ui->priv->have_device_list) {
         ui->priv->have_device_list = TRUE;
@@ -153,46 +229,6 @@ ui_set_scan_devices (SimpleScan *ui, GList *devices)
                            FALSE);
         }
     }
-  
-    have_selection = gtk_combo_box_get_active (GTK_COMBO_BOX (ui->priv->device_combo)) >= 0;
-  
-    /* Remove disappeared devices */
-
-    do {
-        for (have_iter = gtk_tree_model_get_iter_first (ui->priv->device_model, &iter);
-             have_iter;
-             have_iter = gtk_tree_model_iter_next (ui->priv->device_model, &iter)) {
-            gchar *name;
-
-            gtk_tree_model_get (ui->priv->device_model, &iter, 0, &name, -1);
-            for (i = devices; i; i = i->next)
-                if (strcmp (name, ((ScanDevice *) i->data)->name) == 0)
-                    break;
-            g_free (name);
-
-            /* Device was removed */
-            if (i == NULL) {
-                gtk_list_store_remove (GTK_LIST_STORE (ui->priv->device_model), &iter);
-                break;
-            }
-        }
-    } while (have_iter);
-
-    /* Add new devices */
-    for (i = devices; i; i = i->next) {
-        ScanDevice *device = (ScanDevice *) i->data;
-
-        if (!find_scan_device (ui, device->name, &iter)) {
-            gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
-            gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device->name, 1, device->label, -1);
-
-            /* Select this device if none selected */
-            if (!have_selection) {
-                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
-                have_selection = TRUE;
-            }
-        }
-    }
 }
 
 
@@ -219,7 +255,7 @@ ui_set_selected_device (SimpleScan *ui, const gchar *device)
     /* If doesn't exist add with label set to device name */
     if (!find_scan_device (ui, device, &iter)) {
         gtk_list_store_append (GTK_LIST_STORE (ui->priv->device_model), &iter);
-        gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device, 1, device, 2, FALSE, -1);
+        gtk_list_store_set (GTK_LIST_STORE (ui->priv->device_model), &iter, 0, device, 1, device, -1);
     }
 
     gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
@@ -229,13 +265,13 @@ ui_set_selected_device (SimpleScan *ui, const gchar *device)
 static void
 add_default_page (SimpleScan *ui)
 {
-    if (book_get_n_pages (ui->priv->book) > 0)
-        return;
-
-    book_append_page (ui->priv->book,
-                      ui->priv->default_page_width, ui->priv->default_page_height,
-                      ui->priv->default_page_dpi,
-                      ui->priv->default_page_orientation);
+    Page *page;
+  
+    page = book_append_page (ui->priv->book,
+                             ui->priv->default_page_width, ui->priv->default_page_height,
+                             ui->priv->default_page_dpi,
+                             ui->priv->default_page_orientation);
+    book_view_select_page (ui->priv->book_view, page);
 }
 
 
@@ -272,11 +308,34 @@ set_document_hint (SimpleScan *ui, const gchar *document_hint)
 }
 
 
+static void
+set_page_side (SimpleScan *ui, const gchar *document_hint)
+{
+    GtkTreeIter iter;
+
+    if (gtk_tree_model_get_iter_first (ui->priv->page_side_model, &iter)) {
+        do {
+            gchar *d;
+            gboolean have_match;
+
+            gtk_tree_model_get (ui->priv->page_side_model, &iter, 0, &d, -1);
+            have_match = strcmp (d, document_hint) == 0;
+            g_free (d);
+
+            if (have_match) {
+                gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->page_side_combo), &iter);                
+                return;
+            }
+        } while (gtk_tree_model_iter_next (ui->priv->page_side_model, &iter));
+     }
+}
+
+
 static gint
 get_text_dpi (SimpleScan *ui)
 {
     GtkTreeIter iter;
-    gint dpi = 200;
+    gint dpi = DEFAULT_TEXT_DPI;
 
     if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->text_dpi_combo), &iter))
         gtk_tree_model_get (ui->priv->text_dpi_model, &iter, 0, &dpi, -1);
@@ -289,7 +348,7 @@ static gint
 get_photo_dpi (SimpleScan *ui)
 {
     GtkTreeIter iter;
-    gint dpi = 200;
+    gint dpi = DEFAULT_PHOTO_DPI;
 
     if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->photo_dpi_combo), &iter))
         gtk_tree_model_get (ui->priv->photo_dpi_model, &iter, 0, &dpi, -1);
@@ -298,6 +357,19 @@ get_photo_dpi (SimpleScan *ui)
 }
 
 
+static gchar *
+get_page_side (SimpleScan *ui)
+{
+    GtkTreeIter iter;
+    gchar *mode = NULL;
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->page_side_combo), &iter))
+        gtk_tree_model_get (ui->priv->page_side_model, &iter, 0, &mode, -1);
+    
+    return mode;
+}
+
+
 static gchar *
 get_document_hint (SimpleScan *ui)
 {
@@ -325,7 +397,7 @@ scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         dpi = get_text_dpi (ui);
     else
         dpi = get_photo_dpi (ui);
-    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, FALSE);
+    g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, SCAN_SINGLE);
     g_free (device);
     g_free (mode);
 }
@@ -348,18 +420,26 @@ continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     if (ui->priv->scanning) {
         g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
     } else {
-        gchar *device, *mode;
+        gchar *device, *mode, *side;
         gint dpi;
+        ScanType type = SCAN_ADF_BOTH;
 
         device = get_selected_device (ui);
         mode = get_document_hint (ui);
+        side = get_page_side (ui);
         if (strcmp (mode, "text") == 0) 
             dpi = get_text_dpi (ui);
         else
             dpi = get_photo_dpi (ui);
-        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, TRUE);
+        if (strcmp (side, "front") == 0)
+            type = SCAN_ADF_FRONT;
+        else if (strcmp (side, "back") == 0)
+            type = SCAN_ADF_BACK;
+
+        g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, dpi, mode, type);
         g_free (device);
         g_free (mode);
+        g_free (side);
     }
 }
 
@@ -403,7 +483,7 @@ page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
     
     if (page_has_crop (page)) {
         char *crop_name;
-
+      
         // FIXME: Make more generic
         crop_name = page_get_named_crop (page);
         if (crop_name) {
@@ -428,11 +508,68 @@ page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
         name = "no_crop_menuitem";
 
     gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, name)), TRUE);
-    
+    gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (gtk_builder_get_object (ui->priv->builder, "crop_toolbutton")), page_has_crop (page));
+
     ui->priv->updating_page_menu = FALSE;
 }
 
 
+// FIXME: Duplicated from simple-scan.c
+static gchar *
+get_temporary_filename (const gchar *prefix, const gchar *extension)
+{
+    gint fd;
+    gchar *filename, *path;
+    GError *error = NULL;
+
+    /* NOTE: I'm not sure if this is a 100% safe strategy to use g_file_open_tmp(), close and
+     * use the filename but it appears to work in practise */
+
+    filename = g_strdup_printf ("%s-XXXXXX.%s", prefix, extension);
+    fd = g_file_open_tmp (filename, &path, &error);
+    g_free (filename);
+    if (fd < 0) {
+        g_warning ("Error saving page for viewing: %s", error->message);
+        g_clear_error (&error);
+        return NULL;
+    }
+    close (fd);
+
+    return path;
+}
+
+
+static void
+show_page_cb (BookView *view, Page *page, SimpleScan *ui)
+{
+    gchar *path, *uri;
+    GdkScreen *screen;
+    GError *error = NULL;
+  
+    path = get_temporary_filename ("scanned-page", "tiff");
+    if (!path)
+        return;
+
+    screen = gtk_widget_get_screen (GTK_WIDGET (ui->priv->window));
+
+    uri = g_filename_to_uri (path, NULL, NULL);
+    if (page_save_tiff (page, uri, &error))
+        gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+
+    if (error) {
+        ui_show_error (ui,
+                       /* Error message display when unable to preview image */
+                       _("Unable to open image preview application"),
+                       error->message,
+                       FALSE);
+        g_clear_error (&error);
+    }
+
+    g_free (uri);
+    g_free (path);
+}
+
+
 void rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
@@ -458,7 +595,6 @@ rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         return;
     page = book_view_get_selected (ui->priv->book_view);
     page_rotate_right (page);
-    ui->priv->default_page_orientation = page_get_orientation (page);
 }
 
 
@@ -518,12 +654,18 @@ custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
         set_crop (ui, "custom");
 }
 
-void crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+void crop_toolbutton_toggled_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
-crop_toolbutton_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+crop_toolbutton_toggled_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    set_crop (ui, "custom");
+    if (ui->priv->updating_page_menu)
+        return;
+  
+    if (gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (widget)))
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, "custom_crop_menuitem")), TRUE);
+    else
+        gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, "no_crop_menuitem")), TRUE);
 }
 
 
@@ -653,6 +795,7 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     GtkTreeIter iter;
     GtkTreeViewColumn *column;
     const gchar *extension;
+    gchar *directory;
     gint i;
 
     struct
@@ -669,6 +812,13 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
         { NULL, NULL }
     };
 
+    /* Get directory to save to */
+    directory = gconf_client_get_string (ui->priv->client, GCONF_DIR "/save_directory", NULL);
+    if (!directory || directory[0] == '\0') {
+        g_free (directory);
+        directory = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS));
+    }
+
     dialog = gtk_file_chooser_dialog_new (/* Save dialog: Dialog title */
                                           _("Save As..."),
                                           GTK_WINDOW (ui->priv->window),
@@ -678,7 +828,9 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
                                           NULL);
     gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
     gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), directory);
     gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), ui->priv->default_file_name);
+    g_free (directory);
 
     /* Filter to only show images by default */
     filter = gtk_file_filter_new ();
@@ -686,6 +838,7 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
                               /* Save dialog: Filter name to show only image files */
                               _("Image Files"));
     gtk_file_filter_add_pixbuf_formats (filter);
+    gtk_file_filter_add_mime_type (filter, "application/pdf");
     gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
     filter = gtk_file_filter_new ();
     gtk_file_filter_set_name (filter,
@@ -743,6 +896,11 @@ save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 
         g_free (uri);
     }
+
+    gconf_client_set_string (ui->priv->client, GCONF_DIR "/save_directory",
+                             gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog)),
+                             NULL);
+  
     gtk_widget_destroy (dialog);
 }
 
@@ -787,7 +945,10 @@ G_MODULE_EXPORT
 void
 email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0);
+    gchar *mode;
+    mode = get_document_hint (ui);
+    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0, mode);
+    g_free (mode);
 }
 
 
@@ -822,22 +983,14 @@ help_contents_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
     screen = gtk_widget_get_screen (GTK_WIDGET (ui->priv->window));
     gtk_show_uri (screen, "ghelp:simple-scan", gtk_get_current_event_time (), &error);
 
-    if (error != NULL)
+    if (error)
     {
-        GtkWidget *d;
-        /* Error message displayed when unable to launch help browser */
-        const char *message = _("Unable to open help file");
-
-        d = gtk_message_dialog_new (GTK_WINDOW (ui->priv->window),
-                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                    "%s", message);
-        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (d),
-                                                  "%s", error->message);
-        g_signal_connect (d, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-        gtk_window_present (GTK_WINDOW (d));
-
-        g_error_free (error);
+        ui_show_error (ui,
+                       /* Error message displayed when unable to launch help browser */
+                       _("Unable to open help file"),
+                       error->message,
+                      FALSE);
+        g_clear_error (&error);
     }
 }
 
@@ -874,10 +1027,11 @@ about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
                            "program-name", "Simple Scan",
                            "version", VERSION,
                            "comments", description,
+                           "logo-icon-name", "scanner",
                            "authors", authors,
                            "translator-credits", _("translator-credits"),
                            "website", "https://launchpad.net/simple-scan",
-                           "copyright", "Copyright (C) 2009 Canonical Ltd.",
+                           "copyright", "Copyright © 2009 Canonical Ltd.",
                            "license", license,
                            "wrap-license", TRUE,
                            NULL);
@@ -890,28 +1044,31 @@ quit (SimpleScan *ui)
     char *device, *document_type;
     gint i;
 
+    // FIXME: Warn if document with unsaved changes
+
     device = get_selected_device (ui);
     if (device) {
-        gconf_client_set_string(ui->priv->client, "/apps/simple-scan/selected_device", device, NULL);
+        gconf_client_set_string(ui->priv->client, GCONF_DIR "/selected_device", device, NULL);
         g_free (device);
     }
 
     document_type = get_document_hint (ui);
-    gconf_client_set_string(ui->priv->client, "/apps/simple-scan/document_type", document_type, NULL);
+    gconf_client_set_string(ui->priv->client, GCONF_DIR "/document_type", document_type, NULL);
     g_free (document_type);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/text_dpi", get_text_dpi (ui), NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/photo_dpi", get_photo_dpi (ui), NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/text_dpi", get_text_dpi (ui), NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/photo_dpi", get_photo_dpi (ui), NULL);
+    gconf_client_set_string (ui->priv->client, GCONF_DIR "/page_side", get_page_side (ui), NULL);
 
-    gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_width", ui->priv->window_width, NULL);
-    gconf_client_set_int(ui->priv->client, "/apps/simple-scan/window_height", ui->priv->window_height, NULL);
-    gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/window_is_maximized", ui->priv->window_is_maximized, NULL);
+    gconf_client_set_int(ui->priv->client, GCONF_DIR "/window_width", ui->priv->window_width, NULL);
+    gconf_client_set_int(ui->priv->client, GCONF_DIR "/window_height", ui->priv->window_height, NULL);
+    gconf_client_set_bool(ui->priv->client, GCONF_DIR "/window_is_maximized", ui->priv->window_is_maximized, NULL);
 
     for (i = 0; orientation_keys[i].key != NULL && orientation_keys[i].orientation != ui->priv->default_page_orientation; i++);
     if (orientation_keys[i].key != NULL)
-        gconf_client_set_string(ui->priv->client, "/apps/simple-scan/scan_direction", orientation_keys[i].key, NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_width", ui->priv->default_page_width, NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_height", ui->priv->default_page_height, NULL);
-    gconf_client_set_int (ui->priv->client, "/apps/simple-scan/page_dpi", ui->priv->default_page_dpi, NULL);
+        gconf_client_set_string(ui->priv->client, GCONF_DIR "/scan_direction", orientation_keys[i].key, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/page_width", ui->priv->default_page_width, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/page_height", ui->priv->default_page_height, NULL);
+    gconf_client_set_int (ui->priv->client, GCONF_DIR "/page_dpi", ui->priv->default_page_dpi, NULL);
    
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
 }
@@ -962,11 +1119,17 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
 
 
 static void
-page_changed_cb (Page *page, SimpleScan *ui)
+page_size_changed_cb (Page *page, SimpleScan *ui)
 {
     ui->priv->default_page_width = page_get_scan_width (page);
     ui->priv->default_page_height = page_get_scan_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
+}
+
+
+static void
+page_orientation_changed_cb (Page *page, SimpleScan *ui)
+{
     ui->priv->default_page_orientation = page_get_orientation (page);
 }
 
@@ -978,15 +1141,17 @@ page_added_cb (Book *book, Page *page, SimpleScan *ui)
     ui->priv->default_page_height = page_get_scan_height (page);
     ui->priv->default_page_dpi = page_get_dpi (page);
     ui->priv->default_page_orientation = page_get_orientation (page);
-    g_signal_connect (page, "image-changed", G_CALLBACK (page_changed_cb), ui);
+    g_signal_connect (page, "size-changed", G_CALLBACK (page_size_changed_cb), ui);
+    g_signal_connect (page, "orientation-changed", G_CALLBACK (page_orientation_changed_cb), ui);
 }
 
 
 static void
 page_removed_cb (Book *book, Page *page, SimpleScan *ui)
 {
-    /* Ensure always one page */
-    add_default_page (ui);
+    /* If this is the last page add a new blank one */
+    if (book_get_n_pages (ui->priv->book) == 1)
+        add_default_page (ui);
 }
 
 
@@ -1003,13 +1168,11 @@ set_dpi_combo (GtkWidget *combo, gint default_dpi, gint current_dpi)
       { 75,  _("%d dpi (draft)") },
       /* Preferences dialog: Label for resolution value in resolution list (dpi = dots per inch) */
       { 150, _("%d dpi") },
-      { 200, _("%d dpi") },
-      { 400, _("%d dpi") },
+      { 300, _("%d dpi") },
       { 600, _("%d dpi") },
-      { 800, _("%d dpi") },
-      { 1000, _("%d dpi") },
       /* Preferences dialog: Label for maximum resolution in resolution list */      
       { 1200, _("%d dpi (high resolution)") },
+      { 2400, _("%d dpi") },
       { -1, NULL }
     };
     GtkCellRenderer *renderer;
@@ -1052,9 +1215,13 @@ ui_load (SimpleScan *ui)
     GtkBuilder *builder;
     GError *error = NULL;
     GtkCellRenderer *renderer;
-    gchar *device, *document_type, *scan_direction;
+    gchar *device, *document_type, *scan_direction, *page_side;
     gint dpi;
 
+    gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), ICON_DIR);
+
+    gtk_window_set_default_icon_name ("scanner");
+
     builder = ui->priv->builder = gtk_builder_new ();
     gtk_builder_add_from_file (builder, UI_DIR "simple-scan.ui", &error);
     if (error) {
@@ -1077,6 +1244,8 @@ ui_load (SimpleScan *ui)
     ui->priv->preview_scroll = GTK_WIDGET (gtk_builder_get_object (builder, "preview_scrollbar"));
     ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
     ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
+    ui->priv->stop_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "stop_scan_menuitem"));
+    ui->priv->stop_toolbutton = GTK_WIDGET (gtk_builder_get_object (builder, "stop_toolbutton"));
 
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->mode_combo), renderer, TRUE);
@@ -1095,29 +1264,40 @@ ui_load (SimpleScan *ui)
     ui->priv->text_dpi_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->text_dpi_combo));
     ui->priv->photo_dpi_combo = GTK_WIDGET (gtk_builder_get_object (builder, "photo_dpi_combo"));
     ui->priv->photo_dpi_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->photo_dpi_combo));
+    ui->priv->page_side_combo = GTK_WIDGET (gtk_builder_get_object (builder, "page_side_combo"));
+    ui->priv->page_side_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->page_side_combo));
 
-    dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/text_dpi", NULL);
+    dpi = gconf_client_get_int (ui->priv->client, GCONF_DIR "/text_dpi", NULL);
     if (dpi <= 0)
-        dpi = 200;
-    set_dpi_combo (ui->priv->text_dpi_combo, 200, dpi);
-    dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/photo_dpi", NULL);
+        dpi = DEFAULT_TEXT_DPI;
+    set_dpi_combo (ui->priv->text_dpi_combo, DEFAULT_TEXT_DPI, dpi);
+    dpi = gconf_client_get_int (ui->priv->client, GCONF_DIR "/photo_dpi", NULL);
     if (dpi <= 0)
-        dpi = 400;
-    set_dpi_combo (ui->priv->photo_dpi_combo, 400, dpi);
+        dpi = DEFAULT_PHOTO_DPI;
+    set_dpi_combo (ui->priv->photo_dpi_combo, DEFAULT_PHOTO_DPI, dpi);
 
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
     gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, "text", 1);
+    page_side = gconf_client_get_string (ui->priv->client, GCONF_DIR "/page_side", NULL);
+    if (page_side) {
+        set_page_side (ui, page_side);
+        g_free (page_side);
+    }
 
-    device = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/selected_device", NULL);
+    device = gconf_client_get_string (ui->priv->client, GCONF_DIR "/selected_device", NULL);
     if (device) {
         GtkTreeIter iter;
         if (find_scan_device (ui, device, &iter))
             gtk_combo_box_set_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter);
         g_free (device);
     }
-    
-    document_type = gconf_client_get_string (ui->priv->client, "/apps/simple-scan/document_type", NULL);
+
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (ui->priv->page_side_combo), renderer, "text", 1);
+
+    document_type = gconf_client_get_string (ui->priv->client, GCONF_DIR "/document_type", NULL);
     if (document_type) {
         set_document_hint (ui, document_type);
         g_free (document_type);
@@ -1125,6 +1305,7 @@ ui_load (SimpleScan *ui)
 
     ui->priv->book_view = book_view_new ();
     g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
+    g_signal_connect (ui->priv->book_view, "show-page", G_CALLBACK (show_page_cb), ui);
     book_view_set_widgets (ui->priv->book_view,
                            ui->priv->preview_box,
                            ui->priv->preview_area,
@@ -1132,7 +1313,7 @@ ui_load (SimpleScan *ui)
                            GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
 
     /* Find default page details */
-    scan_direction = gconf_client_get_string(ui->priv->client, "/apps/simple-scan/scan_direction", NULL);
+    scan_direction = gconf_client_get_string(ui->priv->client, GCONF_DIR "/scan_direction", NULL);
     ui->priv->default_page_orientation = TOP_TO_BOTTOM;
     if (scan_direction) {
         gint i;
@@ -1140,32 +1321,33 @@ ui_load (SimpleScan *ui)
         if (orientation_keys[i].key != NULL)
             ui->priv->default_page_orientation = orientation_keys[i].orientation;
     }
-    ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_width", NULL);
+    ui->priv->default_page_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_width", NULL);
     if (ui->priv->default_page_width <= 0)
         ui->priv->default_page_width = 595;
-    ui->priv->default_page_height = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_height", NULL);
+    ui->priv->default_page_height = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_height", NULL);
     if (ui->priv->default_page_height <= 0)
         ui->priv->default_page_height = 842;
-    ui->priv->default_page_dpi = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/page_dpi", NULL);
+    ui->priv->default_page_dpi = gconf_client_get_int (ui->priv->client, GCONF_DIR "/page_dpi", NULL);
     if (ui->priv->default_page_dpi <= 0)
         ui->priv->default_page_dpi = 72;
 
     /* Restore window size */
-    ui->priv->window_width = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_width", NULL);
+    ui->priv->window_width = gconf_client_get_int (ui->priv->client, GCONF_DIR "/window_width", NULL);
     if (ui->priv->window_width <= 0)
         ui->priv->window_width = 600;
-    ui->priv->window_height = gconf_client_get_int (ui->priv->client, "/apps/simple-scan/window_height", NULL);
+    ui->priv->window_height = gconf_client_get_int (ui->priv->client, GCONF_DIR "/window_height", NULL);
     if (ui->priv->window_height <= 0)
         ui->priv->window_height = 400;
     g_debug ("Restoring window to %dx%d pixels", ui->priv->window_width, ui->priv->window_height);
     gtk_window_set_default_size (GTK_WINDOW (ui->priv->window), ui->priv->window_width, ui->priv->window_height);
-    ui->priv->window_is_maximized = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/window_is_maximized", NULL);
+    ui->priv->window_is_maximized = gconf_client_get_bool (ui->priv->client, GCONF_DIR "/window_is_maximized", NULL);
     if (ui->priv->window_is_maximized) {
         g_debug ("Restoring window to maximized");
         gtk_window_maximize (GTK_WINDOW (ui->priv->window));
     }
 
-    add_default_page (ui);
+    if (book_get_n_pages (ui->priv->book) == 0)
+        add_default_page (ui);
     book_view_set_book (ui->priv->book_view, ui->priv->book);
 }
 
@@ -1203,13 +1385,8 @@ ui_set_scanning (SimpleScan *ui, gboolean scanning)
 {
     ui->priv->scanning = scanning;
     gtk_widget_set_sensitive (ui->priv->page_delete_menuitem, !scanning);
-}
-
-
-void
-ui_set_have_scan (SimpleScan *ui, gboolean have_scan)
-{
-    //FIXME: gtk_widget_set_sensitive (ui->priv->actions_box, have_scan);
+    gtk_widget_set_sensitive (ui->priv->stop_menuitem, scanning);
+    gtk_widget_set_sensitive (ui->priv->stop_toolbutton, scanning);
 }
 
 
@@ -1250,20 +1427,20 @@ ui_start (SimpleScan *ui)
 
 /* Generated with glib-genmarshal */
 static void
-g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN (GClosure     *closure,
-                                                         GValue       *return_value G_GNUC_UNUSED,
-                                                         guint         n_param_values,
-                                                         const GValue *param_values,
-                                                         gpointer      invocation_hint G_GNUC_UNUSED,
-                                                         gpointer      marshal_data)
+g_cclosure_user_marshal_VOID__STRING_INT_STRING_INT (GClosure     *closure,
+                                                     GValue       *return_value G_GNUC_UNUSED,
+                                                     guint         n_param_values,
+                                                     const GValue *param_values,
+                                                     gpointer      invocation_hint G_GNUC_UNUSED,
+                                                     gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN) (gpointer       data1,
-                                                                gconstpointer  arg_1,
-                                                                gint           arg_2,
-                                                                gconstpointer  arg_3,
-                                                                gboolean       arg_4,
-                                                                gpointer       data2);
-  register GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN callback;
+  typedef void (*GMarshalFunc_VOID__STRING_INT_STRING_INT) (gpointer       data1,
+                                                            gconstpointer  arg_1,
+                                                            gint           arg_2,
+                                                            gconstpointer  arg_3,
+                                                            gint           arg_4,
+                                                            gpointer       data2);
+  register GMarshalFunc_VOID__STRING_INT_STRING_INT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
@@ -1279,13 +1456,13 @@ g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_INT_STRING_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__STRING_INT_STRING_INT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_value_get_string (param_values + 1),
             g_value_get_int (param_values + 2),
             g_value_get_string (param_values + 3),
-            g_value_get_boolean (param_values + 4),
+            g_value_get_int (param_values + 4),
             data2);
 }
 
@@ -1299,8 +1476,8 @@ ui_class_init (SimpleScanClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, start_scan),
                       NULL, NULL,
-                      g_cclosure_user_marshal_VOID__STRING_INT_STRING_BOOLEAN,
-                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_BOOLEAN);
+                      g_cclosure_user_marshal_VOID__STRING_INT_STRING_INT,
+                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT);
     signals[STOP_SCAN] =
         g_signal_new ("stop-scan",
                       G_TYPE_FROM_CLASS (klass),
@@ -1323,8 +1500,8 @@ ui_class_init (SimpleScanClass *klass)
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, email),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
-                      G_TYPE_NONE, 0);
+                      g_cclosure_marshal_VOID__STRING,
+                      G_TYPE_NONE, 1, G_TYPE_STRING);
     signals[QUIT] =
         g_signal_new ("quit",
                       G_TYPE_FROM_CLASS (klass),
@@ -1348,7 +1525,7 @@ ui_init (SimpleScan *ui)
     g_signal_connect (ui->priv->book, "page-added", G_CALLBACK (page_added_cb), ui);
    
     ui->priv->client = gconf_client_get_default();
-    gconf_client_add_dir(ui->priv->client, "/apps/simple-scan", GCONF_CLIENT_PRELOAD_NONE, NULL);
+    gconf_client_add_dir(ui->priv->client, GCONF_DIR, GCONF_CLIENT_PRELOAD_NONE, NULL);
 
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
