@@ -34,10 +34,10 @@ typedef struct
 {
     GObjectClass parent_class;
 
-    void (*start_scan) (SimpleScan *ui, const gchar *device, const gchar *document_type, gboolean continuous, gboolean replace);
+    void (*start_scan) (SimpleScan *ui, const gchar *device, gint dpi, const gchar *document_type, ScanType type);
     void (*stop_scan) (SimpleScan *ui);
     void (*save) (SimpleScan *ui, const gchar *format);
-    void (*email) (SimpleScan *ui);
+    void (*email) (SimpleScan *ui, const gchar *profile);
     void (*quit) (SimpleScan *ui);
 } SimpleScanClass;
 
@@ -64,8 +64,6 @@ void ui_set_selected_device (SimpleScan *ui, const gchar *device);
 
 void ui_set_scanning (SimpleScan *ui, gboolean scanning);
 
-void ui_set_have_scan (SimpleScan *ui, gboolean have_scan);
-
 void ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text, gboolean change_scanner_hint);
 
 void ui_start (SimpleScan *ui);
